# coding: utf-8

"""
    Opal API

    The Opal API is a RESTful API that allows you to interact with the Opal Security platform programmatically.

    The version of the OpenAPI document: 1.0
    Contact: hello@opal.dev
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from opal_security.models.group_remote_info import GroupRemoteInfo
from opal_security.models.group_type_enum import GroupTypeEnum
from opal_security.models.request_configuration import RequestConfiguration
from opal_security.models.risk_sensitivity_enum import RiskSensitivityEnum
from typing import Optional, Set
from typing_extensions import Self

class Group(BaseModel):
    """
    # Group Object ### Description The `Group` object is used to represent a group.  ### Usage Example Update from the `UPDATE Groups` endpoint.
    """ # noqa: E501
    group_id: StrictStr = Field(description="The ID of the group.")
    app_id: Optional[StrictStr] = Field(default=None, description="The ID of the group's app.")
    name: Optional[StrictStr] = Field(default=None, description="The name of the group.")
    description: Optional[StrictStr] = Field(default=None, description="A description of the group.")
    admin_owner_id: Optional[StrictStr] = Field(default=None, description="The ID of the owner of the group.")
    group_leader_user_ids: Optional[List[StrictStr]] = Field(default=None, description="A list of User IDs for the group leaders of the group")
    remote_id: Optional[StrictStr] = Field(default=None, description="The ID of the remote.")
    remote_name: Optional[StrictStr] = Field(default=None, description="The name of the remote.")
    group_type: Optional[GroupTypeEnum] = None
    max_duration: Optional[StrictInt] = Field(default=None, description="The maximum duration for which the group can be requested (in minutes).")
    recommended_duration: Optional[StrictInt] = Field(default=None, description="The recommended duration for which the group should be requested (in minutes). -1 represents an indefinite duration.")
    require_manager_approval: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not access requests to the group require manager approval.")
    require_support_ticket: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not access requests to the group require an access ticket.")
    require_mfa_to_approve: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not to require MFA for reviewers to approve requests for this group.")
    require_mfa_to_request: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not to require MFA for requesting access to this group.")
    auto_approval: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not to automatically approve requests to this group.")
    request_template_id: Optional[StrictStr] = Field(default=None, description="The ID of the associated request template.")
    configuration_template_id: Optional[StrictStr] = Field(default=None, description="The ID of the associated configuration template.")
    group_binding_id: Optional[StrictStr] = Field(default=None, description="The ID of the associated group binding.")
    is_requestable: Optional[StrictBool] = Field(default=None, description="A bool representing whether or not to allow access requests to this group.")
    request_configurations: Optional[List[RequestConfiguration]] = Field(default=None, description="A list of request configurations for this group.")
    request_configuration_list: Optional[List[RequestConfiguration]] = Field(default=None, description="A list of request configurations for this group. Deprecated in favor of `request_configurations`.")
    metadata: Optional[StrictStr] = Field(default=None, description="JSON metadata about the remote group. Only set for items linked to remote systems. See [this guide](https://docs.opal.dev/reference/end-system-objects) for details.")
    remote_info: Optional[GroupRemoteInfo] = None
    custom_request_notification: Optional[Annotated[str, Field(strict=True, max_length=800)]] = Field(default=None, description="Custom request notification sent to the requester when the request is approved.")
    risk_sensitivity: Optional[RiskSensitivityEnum] = Field(default=None, description="The risk sensitivity level for the group. When an override is set, this field will match that.")
    risk_sensitivity_override: Optional[RiskSensitivityEnum] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["group_id", "app_id", "name", "description", "admin_owner_id", "group_leader_user_ids", "remote_id", "remote_name", "group_type", "max_duration", "recommended_duration", "require_manager_approval", "require_support_ticket", "require_mfa_to_approve", "require_mfa_to_request", "auto_approval", "request_template_id", "configuration_template_id", "group_binding_id", "is_requestable", "request_configurations", "request_configuration_list", "metadata", "remote_info", "custom_request_notification", "risk_sensitivity", "risk_sensitivity_override"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Group from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "risk_sensitivity",
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in request_configurations (list)
        _items = []
        if self.request_configurations:
            for _item_request_configurations in self.request_configurations:
                if _item_request_configurations:
                    _items.append(_item_request_configurations.to_dict())
            _dict['request_configurations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in request_configuration_list (list)
        _items = []
        if self.request_configuration_list:
            for _item_request_configuration_list in self.request_configuration_list:
                if _item_request_configuration_list:
                    _items.append(_item_request_configuration_list.to_dict())
            _dict['request_configuration_list'] = _items
        # override the default output from pydantic by calling `to_dict()` of remote_info
        if self.remote_info:
            _dict['remote_info'] = self.remote_info.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Group from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "group_id": obj.get("group_id"),
            "app_id": obj.get("app_id"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "admin_owner_id": obj.get("admin_owner_id"),
            "group_leader_user_ids": obj.get("group_leader_user_ids"),
            "remote_id": obj.get("remote_id"),
            "remote_name": obj.get("remote_name"),
            "group_type": obj.get("group_type"),
            "max_duration": obj.get("max_duration"),
            "recommended_duration": obj.get("recommended_duration"),
            "require_manager_approval": obj.get("require_manager_approval"),
            "require_support_ticket": obj.get("require_support_ticket"),
            "require_mfa_to_approve": obj.get("require_mfa_to_approve"),
            "require_mfa_to_request": obj.get("require_mfa_to_request"),
            "auto_approval": obj.get("auto_approval"),
            "request_template_id": obj.get("request_template_id"),
            "configuration_template_id": obj.get("configuration_template_id"),
            "group_binding_id": obj.get("group_binding_id"),
            "is_requestable": obj.get("is_requestable"),
            "request_configurations": [RequestConfiguration.from_dict(_item) for _item in obj["request_configurations"]] if obj.get("request_configurations") is not None else None,
            "request_configuration_list": [RequestConfiguration.from_dict(_item) for _item in obj["request_configuration_list"]] if obj.get("request_configuration_list") is not None else None,
            "metadata": obj.get("metadata"),
            "remote_info": GroupRemoteInfo.from_dict(obj["remote_info"]) if obj.get("remote_info") is not None else None,
            "custom_request_notification": obj.get("custom_request_notification"),
            "risk_sensitivity": obj.get("risk_sensitivity"),
            "risk_sensitivity_override": obj.get("risk_sensitivity_override")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


